"""
Preprocess the participant characteristics data. This includes the data extracted
from participants' Prolific profiles, as well as data from the prescreening questionnaire.

We distinguish between the participants whose data from the repertory grid study we could
use ("final participants"), and those participants who were paid for the repertory
grid study ("paid grouping participants"). The latter include 3 additional people
who provided non-sensible group labels in the repertory grid study.

Since we cannot provide the non-anonymized data, you cannot run this file yourself.


Required files and folders:
    - Prolific_Exports
    - key_all_prescreening_q_people.xlsx
    - participant_ids_nonsensible_group_labels.csv
    - Prescreening+Questionnaire_Final_November+23,+2022_09.26.csv
    - Prescreening+Questionnaire_Final_Male-Female-Only_November+23,+2022_09.26.csv
    - responses_for_coding_final_full_dataframe.xlsx
    

Output files:
    - get_participant_characteristics_output.txt
    - prescreening_questionnaire_final_participants_only_anonym.csv
    - prescreening_questionnaire_paid_grouping_participants_only_anonym.csv
    - prolific_profile_data_final_participants_only_anonym.csv
    - prolific_profile_data_paid_grouping_participants_only_anonym.csv


@author Nele Albers
Date: November 2023

"""

import glob
import pandas as pd


if __name__ == '__main__':


    # Let's first load our final list of participants
    df_final = pd.read_excel("responses_for_coding_final_full_dataframe.xlsx")
    final_people = df_final["PROLIFIC_PID"].to_list()
    print("Number of final participants:", len(final_people))
    with open("get_participant_characteristics_output.txt", 'w') as f:
        f.write("Number of final participants: " + (str(len(final_people))) + "\n")


    # And we also get the IDs of the participants who completed the grouping
    # questionnaire and were paid for it but provided non-sensible group labels,
    # so we could not use their data. This are 3 participants.
    people_nonsensible = pd.read_csv("participant_ids_nonsensible_group_labels.csv")["Prolific_PID"].to_list()
    print("Number of participants who provided non-sensible group labels:", len(people_nonsensible))
    with open("get_participant_characteristics_output.txt", 'a') as f:
        f.write("Number of participants who provided non-sensible group labels: " + (str(len(people_nonsensible))))


    #### First the characteristics from the prescreening questionnaire
    # We had two questionnaires on Qualtrics, one when we were still trying to
    # balance people according to multiple characteristics, and one when we
    # were only balancing based on male vs. female gender.
    pres1 = pd.read_csv("Prescreening+Questionnaire_Final_November+23,+2022_09.26.csv")
    pres1["Questionnaire"] = ["Original" for j in range(len(pres1))]
    pres2 = pd.read_csv("Prescreening+Questionnaire_Final_Male-Female-Only_November+23,+2022_09.26.csv")
    pres2["Questionnaire"] = ["Gender_Only" for j in range(len(pres2))]

    # Only use completed submissions
    pres1 = pres1[pres1["Finished"] == "TRUE"]
    pres1 = pres1.reset_index(drop=True)
    pres2 = pres2[pres2["Finished"] == "True"]
    pres2 = pres2.reset_index(drop=True)

    # Only use the columns that we can publish
    pres1 = pres1[["PROLIFIC_PID", "IC_1", "IC_2", "IC_3", "IC_4", "IC_5", 'IC_6',
                   "IC_7", "IC_8", "IC_9", "IC_10", "PIV_Weekly_Exercise",
                   "PIV_Smoking_Freq", "ttm_pa", "ttm_smoking",
                   "Quit_Before_24_Hours", "Num_Quit_24h_Last_Y",
                   "Time_Since_Last_Quit", "Questionnaire", "Finished"]]
    pres2 = pres2[["PROLIFIC_PID", "IC_1", "IC_2", "IC_3", "IC_4", "IC_5", 'IC_6',
                   "IC_7", "IC_8", "IC_9", "IC_10", "PIV_Weekly_Exercise",
                   "PIV_Smoking_Freq", "ttm_pa", "ttm_smoking",
                   "Quit_Before_24_Hours", "Num_Quit_24h_Last_Y",
                   "Time_Since_Last_Quit", "Questionnaire", "Finished"]]
    pres = pd.concat([pres2, pres1])
    pres = pres.reset_index(drop=True)

    # And let's only use those people who have completed the grouping session
    # People who did not complete the grouping session were also not paid for the
    # prescreening questionnaire, so we cannot use their data.
    pres = pres[pres["PROLIFIC_PID"].isin(final_people + people_nonsensible)]
    pres = pres.reset_index(drop=True)

    # Some people took part in the original questionnaire and did not pass because
    # some data from their Prolific profile could not be validated in the
    # questionnaire (e.g., weekly exercise amount). Since not all participants from the 
    # original questionnaire had been paid yet when we started the gender-only
    # questionnaire, we could not exclude returned or rejected submissions 
    # for some participant characteristic combinations from participating in the
    # gender-only questionnaire. So some people took the questionnaire again and
    # passed the second time. We now want to keep only the passed submission for
    # these people
    num_appearance = pres["PROLIFIC_PID"].value_counts()
    double_people = [j for j in final_people if num_appearance[j] == 2]
    for person in double_people:
        pres = pres[~((pres["PROLIFIC_PID"] == person) & (pres["Questionnaire"] == "Original"))]
    pres = pres.reset_index(drop = True)
    
    
    #### Now the characteristics from the Prolific profiles
    # First from those people who took part in the original questionnaire
    prolific_file_names_list = glob.glob('Prolific_Exports/prolific_export_age_*.csv')
    df_prolific_list = []
    for file_name in prolific_file_names_list:
        df_prolific_list.append(pd.read_csv(file_name))
    df_prolific_orig = pd.concat(df_prolific_list)

    # And let's only use those people who have completed the grouping session
    df_prolific_orig = df_prolific_orig[df_prolific_orig["Participant id"].isin(final_people + people_nonsensible)]
    df_prolific_orig = df_prolific_orig.reset_index(drop=True)

    # Keep only the first occurrence for each participant
    # Prolific exports were ordered by time, with earlier times higher up.
    # So we then keep the value from the earliest export a participant appears in.
    df_prolific_orig = df_prolific_orig.drop_duplicates(subset="Participant id", 
                                                        keep="first")
    df_prolific_orig = df_prolific_orig.reset_index(drop=True)

    # Next from those people who took part in the gender-only questionnaire
    prolific_file_names_list = glob.glob('Prolific_Exports/prolific_export_*round*.csv')
    df_prolific_list = []
    for file_name in prolific_file_names_list:
        df_prolific_list.append(pd.read_csv(file_name))
    df_prolific_gen = pd.concat(df_prolific_list)

    # And let's only use those people who have completed the grouping session again
    df_prolific_gen = df_prolific_gen[df_prolific_gen["Participant id"].isin(final_people + people_nonsensible)]
    df_prolific_gen = df_prolific_gen.reset_index(drop=True)

    # Keep only the first occurrence for each participant
    # Prolific exports were ordered by time, with earlier times higher up.
    # So we then keep the value from the earliest export a participant appears in.
    df_prolific_gen = df_prolific_gen.drop_duplicates(subset="Participant id", 
                                                      keep="first")
    df_prolific_gen = df_prolific_gen.reset_index(drop=True)

    # Here we do not need to account for those people who took part in both the original
    # and the gender-only questionnaire, because for some time, the Prolific export
    # did not include data on people with returned submissions. So the Prolific exports
    # for the original questionnaire do not include those people whose Prolific data could not be
    # validated and who then returned their submissions.
    df_prolific = pd.concat([df_prolific_gen, df_prolific_orig])
    df_prolific = df_prolific.reset_index(drop=True)


    #### Add anonymized ID
    key = pd.read_excel("key_all_prescreening_q_people.xlsx")
    df_prolific["rand_id"] = [[key.iloc[i]["rand_id"] for i in range(len(key)) if key.iloc[i]["Prolific_ID"] == df_prolific.iloc[p]["Participant id"]][0] for p in range(len(df_prolific))] 
    pres["rand_id"] = [[key.iloc[i]["rand_id"] for i in range(len(key)) if key.iloc[i]["Prolific_ID"] == pres.iloc[p]["PROLIFIC_PID"]][0] for p in range(len(pres))] 
    
    
    #### Use only either the final participants or the paid participants (i.e., 
    # final participants plus the 3 participants with non-sensible answers for
    # grouping labels)
    # And save dataframes to files
    people_subsets = [final_people, final_people + people_nonsensible]
    people_subset_names = ["final_participants_only", "paid_grouping_participants_only"]
    for subset_idx, subset in enumerate(people_subsets):

        pres_sub = pres[pres["PROLIFIC_PID"].isin(subset)]
        pres_sub = pres_sub.reset_index(drop=True)
        df_prolific_sub = df_prolific[df_prolific["Participant id"].isin(subset)]
        df_prolific_sub = df_prolific_sub.reset_index(drop=True)


        # Save dataframes to files
        df_prolific_sub = df_prolific_sub[["rand_id", "Weekly exercise", "Gender",
                                           "Highest education level completed", "Smoking frequency",
                                           "Age"]]
        df_prolific_sub.to_csv("prolific_profile_data_" + people_subset_names[subset_idx] + "_anonym.csv",
                               index=False)

        pres_sub = pres_sub[["rand_id", "IC_1", "IC_2", "IC_3", "IC_4", "IC_5", 'IC_6',
                             "IC_7", "IC_8", "IC_9", "IC_10", "PIV_Weekly_Exercise",
                             "PIV_Smoking_Freq", "ttm_pa", "ttm_smoking",
                             "Quit_Before_24_Hours", "Num_Quit_24h_Last_Y",
                             "Time_Since_Last_Quit", "Questionnaire", "Finished"]]
        pres_sub.to_csv("prescreening_questionnaire_" + people_subset_names[subset_idx] + "_anonym.csv",
                        index=False)
