"""
Preprocess data from raw questionnaire export from the repertory grid study.

Since we cannot provide the non-anonymized data, you cannot run this file yourself.

Participants of this study were asked to divide sets of 3 preparatory activities
into two groups based on either competencies or means for quitting smoking.
Preparatory activities were either about quitting smoking or becoming more 
physically active.


Required files:
    - Competencies_Questionnaire_Export_Raw_2022_11_17_0948.csv
    - key_all_prescreening_q_people.xlsx
    
    
Output files:
    - preprocess_ratings_data_output.txt
    - ratings_data_anonym.csv


@author Nele Albers
Date: November 2023

"""

import pandas as pd


if __name__ == '__main__':
    
    ### First load the questionnaire export
    df = pd.read_csv("Competencies_Questionnaire_Export_Raw_2022_11_17_0948.csv")


    ### Remove first two rows since they contains info on the questions in the
    # questionnaire, not question answers.
    df = df.iloc[2:]
    df = df.reset_index(drop =True)


    ### Only completed responses
    df = df[df["Finished"]=="True"]
    df = df.reset_index(drop=True)


    ### Check passing attention checks.
    # We had to attention check activities. For at least one of them, at least 5 of the 
    # 6 ratings need to have the specified value. For activity 45, this value is 2.
    # For activity 46, this value is 5.
    rating_names_45 = ["45_S_Ends_Group1_R_1", "45_S_Ends_Group2_R_1", 
                       "45_PA_Ends_Group1_R_1", "45_PA_Ends_Group2_R_1",
                       "45_Ease_1", "45_Time_1"]
    answer_45 = "2"
    df["AC_45"] = [sum([1 if df.iloc[person][j] == answer_45 else 0 for j in rating_names_45]) for person in range(len(df))]
    
    rating_names_46 = ["46_S_Ends_Group1_R_1", "46_S_Ends_Group2_R_1", 
                       "46_PA_Ends_Group1_R_1", "46_PA_Ends_Group2_R_1",
                       "46_Ease_1", "46_Time_1"]
    answer_46 = "5"
    df["AC_46"] = [sum([1 if df.iloc[person][j] == answer_46 else 0 for j in rating_names_46]) for person in range(len(df))]

    # People need to pass at least one of these rating AC questions
    df = df[(df["AC_45"] >= 5) | (df["AC_46"]>=5)]


    ### Check if people passed the instruction attention check question
    # This question differed based on whether participants were grouping activities based
    # on means or competencies.
    means_check_correct_answer = "What smokers have to do as part of activities."
    comp_check_correct_answer = "Characteristics (e.g., skills, knowledge, mindset, etc.) that help to successfully quit smoking that are built by doing activities."
    df_means = df[df["Means_check"]==means_check_correct_answer]
    df_comp = df[df["Competencies_check"]==comp_check_correct_answer]


    ### Some labels do not work for subsequent rating of activities
    # We do not consider the data from these three people.
    invalid_labels_comp_g1 = ["Group 1 is taking in information, while group 2 is to use the information to better yourself and start your goal",
                              "GROUP1", "quit 1"]

    df_comp_drop = df_comp[df_comp["S_G1_Label"].isin(invalid_labels_comp_g1)]
    print("Number of people dropped from competencies data for meaningless group labels:", len(df_comp_drop))
    with open("preprocess_ratings_data_output.txt", "w") as f:
        f.write("Number of people dropped from competencies data for meaningless group labels:" + str(len(df_comp_drop)) + "\n")
    df_comp = df_comp[~df_comp["S_G1_Label"].isin(invalid_labels_comp_g1)]
    df_comp = df_comp.reset_index(drop=True)


    ### One person timed out in Prolific, so we do not want to use their data.
    timed_out_answer = "You avoid certain activities that remind you of smoking and you want to avoid gaining a bunch of weight."
    df_comp_drop = df_comp[df_comp["S_Ends_Label1_Exp"] == timed_out_answer]
    print("Number of people dropped from competencies data for having timed out on Prolific:", len(df_comp_drop))
    with open("preprocess_ratings_data_output.txt", "a") as f:
        f.write("Number of people dropped from competencies data for having timed out on Prolific:" + str(len(df_comp_drop)) + "\n")
    df_comp = df_comp[df_comp["S_Ends_Label1_Exp"] != timed_out_answer]
    df_comp = df_comp.reset_index(drop=True)


    ### Next, we need to make sure that people put all 3 activities of the
    # grouping task into a group for the means data (i.e., df_means).
    num_s_act = 26  # number of smoking + mixed activities together
    activities_smoking_0 = ["S_Grouping_Means_0_" + str(i) + "_RANK" for i in range(1, num_s_act + 1)]
    activities_smoking_1 = ["S_Grouping_Means_1_" + str(i) + "_RANK" for i in range(1, num_s_act + 1)]
    activities_smoking = activities_smoking_0 + activities_smoking_1
    num_pa_act = 20  # number of physical activity (PA) + mixed activities together
    activities_pa_0 = ["PA_Grouping_Means_0_" + str(i) + "_RANK" for i in range(1, num_pa_act + 1)]
    activities_pa_1 = ["PA_Grouping_Means_1_" + str(i) + "_RANK" for i in range(1, num_pa_act + 1)]
    activities_pa = activities_pa_0 + activities_pa_1

    # Replace nan for the activity rankings in the groups with 0 so we can
    # compute the sum and not get nan as final result
    for col in activities_smoking:
        df_means[col] = df_means[col].fillna(0)
    df_means["Sum_S"] = [sum([float(df_means.iloc[person][act_smoking]) for act_smoking in activities_smoking]) for person in range(len(df_means))]
    # Now, the sum of the rankings needs to be 4: one activity in each group with rank 1, 
    # and one activity with rank 2
    df_means_drop = df_means[df_means["Sum_S"] != 4]
    print("Number of people dropped from means data for not grouping all smoking activities:", len(df_means_drop))
    with open("preprocess_ratings_data_output.txt", "a") as f:
        f.write("Number of people dropped from means data for not grouping all smoking activities:" + str(len(df_means_drop)) + "\n")
    df_means = df_means[df_means["Sum_S"] == 4]
    df_means = df_means.reset_index(drop=True)

    # Now the same for PA activity grouping
    # Replace nan for the activity rankings in the groups with 0 so we can
    # compute the sum and not get nan as final result
    for col in activities_pa:
        df_means[col] = df_means[col].fillna(0)
    df_means["Sum_PA"] = [sum([float(df_means.iloc[person][act_pa]) for act_pa in activities_pa]) for person in range(len(df_means))]
    # Now, the sum of the rankings needs to be 4: one activity in each group with rank 1, 
    # and one activity with rank 2
    df_means_drop = df_means[df_means["Sum_PA"] != 4]
    print("Number of people dropped from means data for not grouping all physical activity activities:", len(df_means_drop))
    with open("preprocess_ratings_data_output.txt", "a") as f:
        f.write("Number of people dropped from means data for not grouping all physical activity activities:" + str(len(df_means_drop)) + "\n")
    df_means = df_means[df_means["Sum_PA"] == 4]
    df_means = df_means.reset_index(drop=True)


    ### Now the same for df_comp (i.e., the competencies data)
    # Next, we need to make sure that people put all 3 activities of the
    # grouping task into a group
    num_s_act = 26  # number of smoking + mixed activities together
    activities_smoking_0 = ["S_Grouping_Ends_0_" + str(i) + "_RANK" for i in range(1, num_s_act + 1)]
    activities_smoking_1 = ["S_Grouping_Ends_1_" + str(i) + "_RANK" for i in range(1, num_s_act + 1)]
    activities_smoking = activities_smoking_0 + activities_smoking_1
    num_pa_act = 20  # number of PA + mixed activities together
    activities_pa_0 = ["PA_Grouping_Ends_0_" + str(i) + "_RANK" for i in range(1, num_pa_act + 1)]
    activities_pa_1 = ["PA_Grouping_Ends_1_" + str(i) + "_RANK" for i in range(1, num_pa_act + 1)]
    activities_pa = activities_pa_0 + activities_pa_1

    # Replace nan for the activity rankings in the groups with 0 so we can
    # compute the sum and not get nan as final result
    for col in activities_smoking:
        df_comp[col] = df_comp[col].fillna(0)
    df_comp["Sum_S"] = [sum([float(df_comp.iloc[person][act_smoking]) for act_smoking in activities_smoking]) for person in range(len(df_comp))]
    # Now, the sum of the rankings needs to be 4: one activity in each group with rank 1, 
    # and one activity with rank 2
    df_comp_drop = df_comp[df_comp["Sum_S"] != 4]
    print("Number of people dropped from competencies data for not grouping all smoking activities:", len(df_comp_drop))
    with open("preprocess_ratings_data_output.txt", "a") as f:
        f.write("Number of people dropped from competencies data for not grouping all smoking activities:" + str(len(df_comp_drop)) + "\n")
    df_comp = df_comp[df_comp["Sum_S"] == 4]
    df_comp = df_comp.reset_index(drop=True)

    # Now the same for PA activity grouping
    # Replace nan for the activity rankings in the groups with 0 so we can
    # compute the sum and not get nan as final result
    for col in activities_pa:
        df_comp[col] = df_comp[col].fillna(0)
    df_comp["Sum_PA"] = [sum([float(df_comp.iloc[person][act_pa]) for act_pa in activities_pa]) for person in range(len(df_comp))]
    # Now, the sum of the rankings needs to be 4: one activity in each group with rank 1, 
    # and one activity with rank 2
    df_comp_drop = df_comp[df_comp["Sum_PA"] != 4]
    print("Number of people dropped from competencies data for not grouping all physical activity activities:", len(df_comp_drop))
    with open("preprocess_ratings_data_output.txt", "a") as f:
        f.write("Number of people dropped from competencies data for not grouping all physical activity activities:" + str(len(df_comp_drop)) + "\n")
    df_comp = df_comp[df_comp["Sum_PA"] == 4]
    df_comp = df_comp.reset_index(drop=True)
    

    ### Combine the dataframes on means and competencies
    df_all = pd.concat([df_comp, df_means])


    ### Add random identifier for anonymization
    key = pd.read_excel("key_all_prescreening_q_people.xlsx")
    df_all = pd.merge(df_all, key, left_on = "Prolific_PID", 
                      right_on = "Prolific_ID")


    ### Now we want only the data on the difficulty and time ratings of the 
    # activities as well as the free-text response on what makes activities difficult
    questions = ["rand_id", "Difficulty_Response"]
    activity_ratings_stems = ["_Ease_1", "_Time_1", "_Time_Long"]
    for activity_idx in range(1, 45):
        for activity_ratings_stem in activity_ratings_stems:
            questions.append(str(activity_idx) + activity_ratings_stem)


    ### Save final dataframe to file
    df_all = df_all[questions]
    df_all.to_csv("ratings_data_anonym.csv", index=False)
